# !/usr/bin/env python
# -*- coding: utf-8 -*-

"""
    Ce module proclame la bonne parole de sieurs Sam et Max. Puissent-t-ils
    retrouver une totale liberté de pensée cosmique vers un nouvel age
    reminiscent.
"""

__version__ = "0.0.5"

from sibus_lib.DefaultLogger import mylogger
from sibus_lib.MessagingCore import BusElement, BusCore, MessageObject
from sibus_lib.SmartConfig import SmartConfigFile
from sibus_lib.CoreDatabase import CoreDatabase, CoreDBBase

